@@ -249,7 +249,7 @@ static int resize_resident_attribute_value(MFT_RECORD *m, ATTR_RECORD *a,
 static int change_label(ntfs_volume *vol, unsigned long mnt_flags, char *label, BOOL force)
 {
 	ntfs_attr_search_ctx *ctx = NULL;
-	uchar_t *new_label = NULL;
+	ntfschar *new_label = NULL;
 	MFT_RECORD *mrec = NULL;
 	ATTR_RECORD *a;
 	int label_len;
@@ -305,13 +305,13 @@ static int change_label(ntfs_volume *vol, unsigned long mnt_flags, char *label,
 		perror("Unable to convert label string to Unicode");
 		goto err_out;
 	}
-	label_len *= sizeof(uchar_t);
+	label_len *= sizeof(ntfschar);
 	if (label_len > 0x100) {
 		fprintf(stderr, "New label is too long. Maximum %u characters "
 				"allowed. Truncating excess characters.\n",
-				(unsigned)(0x100 / sizeof(uchar_t)));
+				(unsigned)(0x100 / sizeof(ntfschar)));
 		label_len = 0x100;
-		new_label[label_len / sizeof(uchar_t)] = cpu_to_le16(L'\0');
+		new_label[label_len / sizeof(ntfschar)] = cpu_to_le16(L'\0');
 	}
 	if (a) {
 		if (resize_resident_attribute_value(mrec, a, label_len)) {
@@ -345,7 +345,8 @@ static int change_label(ntfs_volume *vol, unsigned long mnt_flags, char *label,
 		a->reservedR = 0;
 	}
 	memcpy((u8*)a + le16_to_cpu(a->value_offset), new_label, label_len);
-	if (ntfs_mft_record_write(vol, (MFT_REF)FILE_Volume, mrec)) {
+	if (!opts.noaction &&
+			ntfs_mft_record_write(vol, (MFT_REF)FILE_Volume, mrec)) {
 		perror("Error writing MFT Record to disk");
 		goto err_out;
 	}
@@ -377,8 +378,11 @@ int main(int argc, char **argv)
 
 	utils_set_locale();
 
-	//XXX need to set and get mount flags
-	vol = utils_mount_volume (opts.device, 0, opts.force);
+	if (!opts.label)
+		opts.noaction++;
+
+	vol = utils_mount_volume (opts.device, opts.noaction ? MS_RDONLY : 0,
+			opts.force);
 	if (!vol)
 		return 1;
 
