@@ -1,7 +1,7 @@
 /**
  * utils.c - Part of the Linux-NTFS project.
  *
- * Copyright (c) 2002-2003 Richard Russon
+ * Copyright (c) 2002-2004 Richard Russon
  * Copyright (c) 2003-2004 Anton Altaparmakov
  * Copyright (c) 2003 Lode Leroy
  *
@@ -567,7 +567,7 @@ int utils_attr_get_name (ntfs_volume *vol, ATTR_RECORD *attr, char *buffer, int
 
 	name    = NULL;
 	namelen = attr->name_length;
-	if (ntfs_ucstombs ((uchar_t *)((char *)attr + attr->name_offset),
+	if (ntfs_ucstombs ((ntfschar *)((char *)attr + attr->name_offset),
 	    namelen, &name, namelen) < 0) {
 		Eprintf ("Couldn't translate attribute name to current locale.\n");
 		// <UNKNOWN>?
@@ -721,7 +721,7 @@ ntfs_inode * utils_pathname_to_inode (ntfs_volume *vol, ntfs_inode *parent, cons
 	char *p, *q;
 	ntfs_inode *ni;
 	ntfs_inode *result  = NULL;
-	uchar_t    *unicode = NULL;
+	ntfschar    *unicode = NULL;
 	char       *ascii   = NULL;
 
 	if (!vol || !pathname) {
@@ -771,6 +771,7 @@ ntfs_inode * utils_pathname_to_inode (ntfs_volume *vol, ntfs_inode *parent, cons
 		if (ni != parent)
 			ntfs_inode_close (ni);
 
+		inum = MREF (inum);
 		ni = ntfs_inode_open (vol, inum);
 		if (!ni) {
 			Eprintf ("Cannot open inode %llu: %s.\n",
@@ -873,28 +874,41 @@ int utils_is_metadata (ntfs_inode *inode)
  * Display a block of memory in a tradition hex-dump manner.
  * Optionally the ascii part can be turned off.
  */
-void utils_dump_mem (u8 *buf, int start, int length, int ascii)
+void utils_dump_mem (u8 *buf, int start, int length, int flags)
 {
-	int off, i, s, e;
+	int off, i, s, e, col;
 
 	s =  start                & ~15;	// round down
 	e = (start + length + 15) & ~15;	// round up
 
 	for (off = s; off < e; off += 16) {
+		col = 30;
+		if (flags & DM_RED)
+			col += 1;
+		if (flags & DM_GREEN)
+			col += 2;
+		if (flags & DM_BLUE)
+			col += 4;
+		if (flags & DM_INDENT)
+			printf ("\t");
+		if (flags & DM_BOLD)
+			printf ("[01m");
+		if (flags & (DM_RED | DM_BLUE | DM_GREEN | DM_BOLD))
+			printf ("[%dm", col);
 		if (off == s)
 			printf("%6.6x ", start);
 		else
 			printf("%6.6x ", off);
 
 		for (i = 0; i < 16; i++) {
-			if (i == 8)
+			if ((i == 8) && (!(flags & DM_NO_DIVIDER)))
 				printf (" -");
 			if (((off+i) >= start) && ((off+i) < (start+length)))
 				printf (" %02X", buf[off+i]);
 			else
 				printf ("   ");
 		}
-		if (ascii) {
+		if (!(flags & DM_NO_ASCII)) {
 			printf ("  ");
 			for (i = 0; i < 16; i++) {
 				if (((off+i) < start) || ((off+i) >= (start+length)))
@@ -905,6 +919,8 @@ void utils_dump_mem (u8 *buf, int start, int length, int ascii)
 					printf (".");
 			}
 		}
+		if (flags & (DM_RED | DM_BLUE | DM_GREEN | DM_BOLD))
+			printf ("[0m");
 		printf ("\n");
 	}
 }
